#include "../Script.hpp"
#include "../UserData/UserDataFunctionType.hpp"

namespace Porygon{
    inline bool ScriptType::CanBeIndexedWith(const ScriptType *) const{
        return false;
    }

    shared_ptr<const ScriptType> ScriptType::BoolType = make_shared<ScriptType>(TypeClass::Bool);
    shared_ptr<const ScriptType> ScriptType::NilType = make_shared<ScriptType>(TypeClass::Nil);
    shared_ptr<const NumericScriptType> NumericScriptType::AwareInt = make_shared<NumericScriptType>(true, false);
    shared_ptr<const NumericScriptType> NumericScriptType::AwareFloat = make_shared<NumericScriptType>(true, true);
    shared_ptr<const NumericScriptType> NumericScriptType::Unaware = make_shared<NumericScriptType>(false, false);
    shared_ptr<const StringScriptType> StringScriptType::Dynamic = make_shared<StringScriptType>(false, 0);

    shared_ptr<const ScriptType> ScriptType::GetIndexedType(const ScriptType*) const{
        return make_shared<ScriptType>(TypeClass::Error);
    }

    extern "C"{
        ScriptType* CreateScriptType(Porygon::TypeClass t){
            return new ScriptType(t);
        }

        const ScriptType* CreateNumericScriptType(bool isAware, bool isFloat){
            return new NumericScriptType(isAware, isFloat);
        }

        ScriptType* CreateStringScriptType(bool knownAtBind, uint32_t hash){
            return new StringScriptType(knownAtBind, hash);
        }

        ScriptType* CreateUserDataFunctionScriptType(ScriptType* returnType, ScriptType* parameters[], size_t parameterCount){
            vector<shared_ptr<const ScriptType>> vector(parameterCount);
            for (size_t i = 0; i < parameterCount; i++){
                vector[i] = shared_ptr<const ScriptType>(parameters[i]);
            }
            auto option = new UserData::UserDataFunctionOption(shared_ptr<const ScriptType>(returnType), vector);
            auto type = new GenericFunctionScriptType();
            type->RegisterFunctionOption(option);
            return type;
        }

        TypeClass GetTypeClass(ScriptType* t){
            return t->GetClass();
        }
    }
}

